==> ba-react-fetch-api-0.9.6.tgz: package/FetchAPI/helpers/with-optimistic.js <==
// @flow
import _ from 'lodash';
import {TFetchedObject, TFetchedArray} from 'flow-typed';

export const actionTypes = {
	optimisticFailed : 'FetchAPI@OPTIMISTIC_FAILED',
	optimisticSuccess: 'FetchAPI@OPTIMISTIC_SUCCESS',
	optimisticUpdate : 'FetchAPI@OPTIMISTIC_UPDATE',
};

type TUpdateFunction = <T: TFetchedObject | TFetchedArray>(state: T) => T;

const
	createAction = ({action, dispatch}) => async (params: {
		body: {},
		callback?: (err: ?{}, response?: mixed) => void,
		commit?: <T: TFetchedObject | TFetchedArray, R: mixed>(state: T, response?: R) => T,
		params: {},
		update?: TUpdateFunction | {[string]: TUpdateFunction},
		updateKeys: Array<string>,
		validate?: (response?: mixed) => void,
	}) => {
		const

			// Suppress flow.js type violation errors with _.identity
			callback = params.callback || _.identity,
			commit = params.commit || _.identity,
			update = params.update || _.identity,
			validate = params.validate || _.identity;

		let
			response = null;

		// Run optimistic update
		dispatch({type: actionTypes.optimisticUpdate, update, updateKeys: params.updateKeys});

		try {
			response = await dispatch(action(
				params.params,
				{body: JSON.stringify(params.body)}
			));

			// Custom validation method for use case like status === 200, but API RESULT is {result: 'Failed'}
			validate(response);
		} catch(err) {
			dispatch({err, response, type: actionTypes.optimisticFailed, updateKeys: params.updateKeys});
			callback(err, response);
			throw err;
		}

		dispatch({commit, response, type: actionTypes.optimisticSuccess, updateKeys: params.updateKeys});
		callback(null, response);
		return response;
	};

export default (entityId: string, entity: {}) => _.extend(entity, {
	helpers: {
		optimisticDelete(params) {
			createAction({
				action  : _.get(this.actions, `${entityId}.delete`),
				dispatch: this.dispatch,
				getState: this.getState,
			})(params);
		},

		optimisticPost(params) {
			createAction({
				action  : _.get(this.actions, `${entityId}.post`),
				dispatch: this.dispatch,
				getState: this.getState,
			})(params);
		},

		optimisticPut(params) {
			createAction({
				action  : _.get(this.actions, `${entityId}.put`),
				dispatch: this.dispatch,
				getState: this.getState,
			})(params);
		},
	},
});

==> ba-react-fetch-api-0.12.1.tgz: package/FetchAPI/helpers/with-optimistic.js <==
// @flow
import _ from 'lodash';
import {TFetchedObject, TFetchedArray} from 'flow-typed';

export const actionTypes = {
	optimisticFailed : 'FetchAPI@OPTIMISTIC_FAILED',
	optimisticSuccess: 'FetchAPI@OPTIMISTIC_SUCCESS',
	optimisticUpdate : 'FetchAPI@OPTIMISTIC_UPDATE',
};

type TUpdateFunction = <T: TFetchedObject | TFetchedArray>(state: T) => T;

const
	createAction = ({action, dispatch, getState, get}: {
		action: Function,
		dispatch: Function,
		get?: Function,
		getState: Function,
	}) => async (params: {
		body: {},
		callback?: (err: ?{}, response?: mixed) => void,
		commit?: <T: TFetchedObject | TFetchedArray, R: mixed>(state: T, response?: R) => T,
		getEtagFromState?: Function,
		params: {},
		update?: TUpdateFunction | {[string]: TUpdateFunction},
		updateKeys: Array<string>,
		validate?: (response?: mixed) => void,
		withEtag?: boolean
	}) => {
		const

			// Suppress flow.js type violation errors with _.identity
			callback = params.callback || _.identity,
			commit = params.commit || _.identity,
			update = params.update || _.identity,
			validate = params.validate || _.identity;

		let
			response = null;

		// Run optimistic update
		dispatch({type: actionTypes.optimisticUpdate, update, updateKeys: params.updateKeys});

		try {
			if(params.withEtag && get && _.isFunction(get))

				// Get entity for further receive etag
				await dispatch(get(params.params));

			response = await dispatch(action(
				params.params,

				{
					body: JSON.stringify(params.body),

					...params.withEtag

						// WARN Treat etag more careful in concern of concurrent data updates
						? {headers: {'If-Match': _.result(params, 'getEtagFromState')(getState())}}

						: {},
				}
			));

			// Custom validation method for use case like status === 200, but API RESULT is {result: 'Failed'}
			validate(response);
		} catch(err) {
			dispatch({err, response, type: actionTypes.optimisticFailed, updateKeys: params.updateKeys});
			callback(err, response);
			throw err;
		}

		dispatch({commit, response, type: actionTypes.optimisticSuccess, updateKeys: params.updateKeys});
		callback(null, response);
		return response;
	};

export default (entityId: string, entity: {}) => _.extend(entity, {
	helpers: {
		optimisticDelete(params) {
			createAction({
				action  : _.get(this.actions, `${entityId}.delete`),
				dispatch: this.dispatch,
				getState: this.getState,
			})(params);
		},

		optimisticPost(params) {
			createAction({
				action  : _.get(this.actions, `${entityId}.post`),
				dispatch: this.dispatch,
				getState: this.getState,
			})(params);
		},

		optimisticPut(params) {
			createAction({
				action  : _.get(this.actions, `${entityId}.put`),
				dispatch: this.dispatch,
				get     : _.get(this.actions, `${entityId}.get`),
				getState: this.getState,
			})(params);
		},
	},
});
